from dune.perftool.generation import (preamble,
                                      symbol,
                                      temporary_variable,
                                      )
from dune.perftool.pdelab.quadrature import (name_quadrature_position,
                                             quadrature_preamble,
                                             )


@preamble
def define_geometry(name):
    return "auto {} = eg.geometry();".format(name)


@symbol
def name_geometry():
    define_geometry("geo")
    return "geo"


@symbol
def type_geometry():
    return 'EG'


@preamble
def define_dimension(name):
    geo = type_geometry()
    return 'const int {} = {}::Entity::dimension;'.format(name, geo)


@symbol
def name_dimension():
    define_dimension("dim")
    return "dim"


@symbol
def type_jacobian_inverse_transposed():
    geo = type_geometry()
    return "typename {}::Geometry::JacobianInverseTransposed".format(geo)


@preamble
def define_jacobian_inverse_transposed_temporary(name, shape, shape_impl):
    t = type_jacobian_inverse_transposed()
    return "{} {};".format(t,
                           name,
                           )


def define_jacobian_inverse_transposed(name):
    dim = name_dimension()
    temporary_variable(name, decl_method=define_jacobian_inverse_transposed_temporary, shape=(dim, dim))
    geo = name_geometry()
    pos = name_quadrature_position()
    return quadrature_preamble("{} = {}.jacobianInverseTransposed({});".format(name,
                                                                               geo,
                                                                               pos,
                                                                               )
                               )


@symbol
def name_jacobian_inverse_transposed():
    define_jacobian_inverse_transposed("J")
    return "J"
